// ====================================================================================================
// An simple example to calculate the value of the inputted expression, by using Vnano Engine.
//   See: https://www.vcssl.org/en-us/vnano/doc/tutorial/feature
// 
// Vnano Engine を用いて、入力された計算式の値を求める、簡単なサンプルコードです。
//   参照: https://www.vcssl.org/ja-jp/vnano/doc/tutorial/feature
// 
// License: CC0 (Copyright Free)
// ====================================================================================================

import org.vcssl.nano.VnanoEngine;
import org.vcssl.nano.VnanoException;
import java.util.Map;
import java.util.HashMap;
import java.util.Scanner;

public class ExampleApp2 {
    public static void main(String[] args) throws VnanoException {

        // Create a scripting engine of Vnano (= Vnano Engine).
        // Vnano Engine のインスタンスを生成
        VnanoEngine engine = new VnanoEngine();

        // Set an option, to handle all numeric literals as "float" (=double) type.
        // (Useful when calculate expressions, but don't enable when run scripts.)
        // 整数リテラルを float (=double) 型と見なすオプションを有効化
        // (式の計算用途に便利ですが、スクリプトの実行用途には適しません)
        Map<String, Object> optionMap = new HashMap<String, Object>();
        optionMap.put("EVAL_INT_LITERAL_AS_FLOAT", true);
        engine.setOptionMap(optionMap);

        // Get an expression from the user.
        // ユーザーに式を入力してもらう
        System.out.println("Input an expression, e.g.:  1.2 + 3.4 * 5.6");
        Scanner scanner = new Scanner(System.in);
        String expression = scanner.nextLine();

        // Append ";" at the end of the expression, if it does not exist.
        // 入力内容が「 ; 」で終わっていない場合は、末尾に付ける
        if (!expression.trim().endsWith(";")) {
            expression += ";";
        }

        // Execute the inputted expression by Vnano Engine.
        // Vnano Engine で式の値を計算し、結果を表示
        double result = (Double)engine.executeScript(expression);
        System.out.println("result: " + result);
    }
}
