// ====================================================================================================
// An example accessing to a field/method from the inputted expression.
//   See: https://www.vcssl.org/en-us/vnano/doc/tutorial/feature
// 
// 入力された式の中から、フィールド値を参照したり、メソッドを呼び出すサンプルコードです。
//   参照: https://www.vcssl.org/ja-jp/vnano/doc/tutorial/feature
// 
// License: CC0 (Copyright Free)
// ====================================================================================================

import org.vcssl.nano.VnanoEngine;
import org.vcssl.nano.VnanoException;
import java.util.Map;
import java.util.HashMap;
import java.util.Scanner;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

public class ExampleApp3 {

    public static class AnyClass {

        // A field and a method to be accessed from
        // an expression/script runs on Vnano Engine.
        // Vnano Engine 上で実行される式やスクリプト内から
        // アクセスしたいフィールド/メソッド
        public double x = 3.4;
        public double f(double arg) {
            return arg * 5.6;
        }
    }

    public static void main(String[] args) throws VnanoException, NoSuchFieldException, NoSuchMethodException {

        // Display the result.
        // Vnano Engine のインスタンスを生成
        VnanoEngine engine = new VnanoEngine();

        // Connect a field/method of "AnyClass" class to Vnano Engine.
        // 任意のクラス（ここでは上のAnyClass）のフィールド/メソッドを Vnano Engine に接続
        Field field = AnyClass.class.getField("x");
        Method method = AnyClass.class.getMethod("f", double.class);
        AnyClass anyClassInstance = new AnyClass();
        engine.connectPlugin("x", new Object[]{ field, anyClassInstance });
        engine.connectPlugin("f", new Object[]{ method, anyClassInstance });

    	// For staric field/method, you can connect it more simple as follows:
        // staric なフィールド/メソッドは、以下のようにインスタンスなしで接続できます:
    	// -----
        // Field field = AnyClass.class.getField("x");
        // Method method = AnyClass.class.getMethod("f", double.class);
        // engine.connectPlugin("x", field);
        // engine.connectPlugin("f", method);

        // Set an option, to handle all numeric literals as "float" (=double) type.
        // (Useful when calculate expressions, but don't enable when run scripts.)
        // 整数リテラルを float (=double) 型と見なすオプションを有効化
        // (式の計算用途に便利ですが、スクリプトの実行用途には適しません)
        Map<String, Object> optionMap = new HashMap<String, Object>();
        optionMap.put("EVAL_INT_LITERAL_AS_FLOAT", true);
        engine.setOptionMap(optionMap);

        // Get an expression from the user.
        // ユーザーに式を入力してもらう
        System.out.println("Input an expression, e.g.:  1.2 + 3.4 * 5.6");
        Scanner scanner = new Scanner(System.in);
        String expression = scanner.nextLine();

        // Append ";" at the end of the expression, if it does not exist.
        // 入力内容が「 ; 」で終わっていない場合は、末尾に付ける
        if (!expression.trim().endsWith(";")) {
            expression += ";";
        }

        // Execute the inputted expression by Vnano Engine.
        // Vnano Engine で式の値を計算し、結果を表示
        double result = (Double)engine.executeScript(expression);
        System.out.println("result: " + result);
    }
}
